(function() {
  var AWS, Promise, Queue, _, debug, queueConfigDefaults, retry, tryParse,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require("lodash");

  AWS = require('aws-sdk');

  Promise = require("bluebird");

  retry = require("bluebird-retry");

  debug = require("debug")("long-task-queue-reader:queue");

  tryParse = function(it) {
    try {
      return JSON.parse(it);
    } catch (undefined) {}
  };

  queueConfigDefaults = {
    VisibilityTimeout: "240",
    MessageRetentionPeriod: "28800"
  };

  module.exports = Queue = (function() {
    function Queue(options) {
      this._push = bind(this._push, this);
      this.create = bind(this.create, this);
      this._setQueueUrl = bind(this._setQueueUrl, this);
      this.initialize = bind(this.initialize, this);
      this.queueName = options.queueName || options.name;
      this.client = this._buildClient(options);
      this.attributes = _.defaults(options.config, queueConfigDefaults);
    }

    Queue.prototype.initialize = function() {
      return Promise.map([this.queueName, this._poisonQueueName()], this.create).then(this._setQueueUrl);
    };

    Queue.prototype._setQueueUrl = function() {
      return this._queueUrl(this.queueName).get("QueueUrl").then((function(_this) {
        return function(queueUrl1) {
          _this.queueUrl = queueUrl1;
        };
      })(this));
    };

    Queue.prototype.create = function(queueName) {
      return this.client.createQueueAsync({
        QueueName: queueName,
        Attributes: this.attributes
      })["catch"]((function(_this) {
        return function(e) {
          if (e.code !== "QueueAlreadyExists") {
            throw e;
          }
        };
      })(this));
    };

    Queue.prototype.sendToPoison = function(message) {
      return this.pushPoison(message.Body);
    };

    Queue.prototype.messages = function(opts) {
      if (opts == null) {
        opts = {};
      }
      return this.client.receiveMessageAsync({
        AttributeNames: ["All"],
        MaxNumberOfMessages: opts.maxMessages || 1,
        MessageAttributeNames: [],
        QueueUrl: this.queueUrl,
        VisibilityTimeout: opts.visibilityTimeout || 120,
        WaitTimeSeconds: opts.waitingTime || 0
      }).then(function(data) {
        if (!data.Messages) {
          return [];
        }
        debug("Received Messages: %o", _.map(data.Messages, 'Body'));
        return data.Messages.map((function(_this) {
          return function(it) {
            return _.update(it, "Body", tryParse);
          };
        })(this));
      });
    };

    Queue.prototype.update = function(timeout, arg) {
      var Body, MessageId, ReceiptHandle;
      MessageId = arg.MessageId, ReceiptHandle = arg.ReceiptHandle, Body = arg.Body;
      debug("Updating [timeout: " + timeout + ", messageId: " + MessageId + ", popReceipt: " + ReceiptHandle + ", messageText: " + (JSON.stringify(Body)) + "]");
      return this.client.changeMessageVisibilityAsync({
        ReceiptHandle: ReceiptHandle,
        QueueUrl: this.queueUrl,
        VisibilityTimeout: timeout
      }).tap(function(message) {
        return debug("Updated message: %o", message);
      });
    };

    Queue.prototype.remove = function(arg) {
      var MessageId, ReceiptHandle;
      MessageId = arg.MessageId, ReceiptHandle = arg.ReceiptHandle;
      debug("Removing message: [messageId: " + MessageId + ", popReceipt: " + ReceiptHandle + "]");
      return retry((function(_this) {
        return function() {
          return _this.client.deleteMessageAsync({
            QueueUrl: _this.queueUrl,
            ReceiptHandle: ReceiptHandle
          });
        };
      })(this)).tap(function() {
        return debug("Removed messageId: " + MessageId);
      });
    };

    Queue.prototype.push = function(message) {
      return this._push(this.queueUrl, message);
    };

    Queue.prototype.pushPoison = function(message) {
      return this._push(this._poisonQueueUrl(), message);
    };

    Queue.prototype._push = function(queueUrl, message) {
      return this.client.sendMessageAsync({
        DelaySeconds: 0,
        MessageAttributes: {},
        MessageBody: JSON.stringify(message),
        QueueUrl: queueUrl
      });
    };

    Queue.prototype._poisonQueueUrl = function() {
      return this._toPoison(this.queueUrl);
    };

    Queue.prototype._poisonQueueName = function() {
      return this._toPoison(this.queueName);
    };

    Queue.prototype._toPoison = function(it) {
      return it + "-poison";
    };

    Queue.prototype._queueUrl = function(queueName) {
      return this.client.getQueueUrlAsync({
        QueueName: queueName
      });
    };

    Queue.prototype._buildClient = function(arg) {
      var access, config, ref, region, secret;
      access = arg.access, secret = arg.secret, region = (ref = arg.region) != null ? ref : "us-east-1";
      config = new AWS.Config({
        accessKeyId: access,
        secretAccessKey: secret,
        region: region
      });
      return Promise.promisifyAll(new AWS.SQS(config), {
        filter: function(functionName) {
          return !_(functionName).endsWith("Async");
        }
      });
    };

    return Queue;

  })();

}).call(this);
